<?php
namespace Controller;

use Zeedhi\Framework\DTO\Request;
use Traits\TResponse;
use Service\CustomizationFilesService;
use Util\Utilities;

class CustomizationFilesController
{
    protected $CustomizationFilesService;

    use TResponse;

   public function __construct(CustomizationFilesService $CustomizationFilesService)
   {
       $this->CustomizationFilesService = $CustomizationFilesService;
   }

   public function getPais()
   {
        try {
            $data = $this->CustomizationFilesService->getPais();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
   }

   public function getEstado(Request $request)
   {
        try {
            $params = $request->getParameters();
            $cdpais = array_key_exists('CDPAIS', $params) ? $params['CDPAIS'] : '';
            $data = $this->CustomizationFilesService->getEstado($cdpais);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
   }

   public function getMunicipio(Request $request)
   {
        try {
            $params = $request->getParameters();
            $sgestado = array_key_exists('SGESTADO', $params) ? $params['SGESTADO'] : '';
            $data = $this->CustomizationFilesService->getMunicipio($sgestado);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
   }

   public function getUseLicense()
   {
        try {
            $data = $this->CustomizationFilesService->getUseLicense();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
   }

   public function downloadPlanilhaPadrao()
   {
        $this->CustomizationFilesService->downloadPlanilhaPadrao();
   }

   public function downloadPlanilhaPadraoInPER()
   {
        $this->CustomizationFilesService->downloadPlanilhaPadraoInPER();
   }

   public function validaArquivoPersonalizado(Request $request)
   {
       try {
           $file = $request->getParameter('FILE');
           $actions = $this->CustomizationFilesService->validaArquivoPersonalizado($file);
           $this->sendResponse($actions, 200);
       } catch (\Exception $ex) {
           $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
       }
   }

   public function saveCustomizationFiles(Request $request) {
        try{
            $rows = $request->getParameters(); 
            $response = [
                'row'    => [],
                'rowSave' => 0
            ];
            foreach ($rows as $row) {
                if (isset($row['erroEncontrado'])) {
                    $response['row'][] = $row;
                } else {
                    $cdlocaluso = $row['CDLOCALUSO'];
                    $nrlicencauso = Utilities::zeroLeftPadding($row['CDLOCALUSO'], 10);
                    $nmfantluso = $row['NMFANTLUSO'];
                    $nmrazsocluso = $row['NMRAZSOCLUSO'];
                    $idtpijurluso = $row['IDTPIJURLUSO'];
                    $nrinsjurluso = $row['NRINSJURLUSO'];
                    $nrinsestluso = $row['NRINSESTLUSO'];
                    $nrinsmuniluso = $row['NRINSMUNILUSO'];
                    $nmcontluso = $row['NMCONTLUSO'];
                    $nmdeptoluso = $row['NMDEPTOLUSO'];
                    $idfaturafilisep = 'N';
                    $dsendeluso = $row['DSENDELUSO'];
                    $nmbairluso = $row['NMBAIRLUSO'];
                    $nrcepluso = $row['NRCEPLUSO'];
                    $cdpais = $row['NMPAIS'];
                    $sgestado = $row['SGESTADO'];
                    $cdmunicipio = $row['NMMUNICIPIO'];
                    $nrteleluso = $row['NRTELELUSO'];
                    $nrramalluso = empty($row['NRRAMALLUSO']) ? null : $row['NRRAMALLUSO'];
                    $nrfaxluso = empty($row['NRFAXLUSO']) ? null : $row['NRFAXLUSO'];
                    $dsemailcomlu = empty($row['DSEMAILCOMLU']) ? null : $row['DSEMAILCOMLU'];
                    $cdlinpclie = $row['CDLINPCLIE'];
                    $idmodproc = 'F';
                    $idlocaproc = 'F';
                    $this->CustomizationFilesService->saveCustomizationFiles($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                    $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                    $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                    $cdlinpclie, $nrlicencauso, $idmodproc, $idlocaproc, $idtpijurluso, $rows);
                    $response['rowSave'] += 1;
                }
            }
            $this->sendResponse($response, 200);
        } catch(\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inativarLocaisUsoArquivoPersonalizado(Request $request) {
        try{
            $rows = $request->getParameters();
            $arrayId = [];
            foreach ($rows as $row) {
                $cdlocaluso = $row['CDLOCALUSO'];
                $arrayId[] = Utilities::zeroLeftPadding($cdlocaluso, 10);
            }
            $this->CustomizationFilesService->inativarLocaisUsoArquivoPersonalizado($arrayId);
        } catch(\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}