<?php
namespace Service;

use Factory\FaturamentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Traits\TResponse;
use Service\BilletIssued;

class FaturamentoService
{
    use TResponse;

    protected $FaturamentoFactory;
    protected $EntityManager;
    protected $Environment;
    protected $newCode;
    protected $BilletIssued;
    protected $nrParc;
    protected $connection;

    public function __construct(FaturamentoFactory $FaturamentoFactory, Environment $Environment, NewCode $newCode, EntityManager $EntityManager, BilletIssued $BilletIssued)
    {
        $this->connection                   = $EntityManager->getConnection();
        $this->FaturamentoFactory           = $FaturamentoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->BilletIssued                 = $BilletIssued;
        $this->nrParc                       = $this->FaturamentoFactory->getNrparcnegocio($_SESSION["_sf2_attributes"]["cdCiente"]);
    }

    public function getFaturamento($parametro)
    {
        $dynamicFilter = !empty($parametro['dynamic_filter'])  ? $parametro['dynamic_filter'] : null;
        $dynamicFilter = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
       // debugApi([$dynamicFilter]);

        $dtinicompet = isset($dynamicFilter["DTINICOMPET"][0]) ? str_replace('-', '/', $dynamicFilter["DTINICOMPET"][0]['value']) : null;
        $dtfincompet = isset($dynamicFilter["DTFINCOMPET"][0]) ? str_replace('-', '/', $dynamicFilter["DTFINCOMPET"][0]['value']) : null;
        
        /*$dtinicompetFilter = '';  Foram unificados os filtros de data inicial e final
        $dtfincompetFilter = '';
        $dtinicompetFilterMed = '';
        $dtfincompetFilterMed = '';
        $qtMesesExibicao = -2;  Retirada a função do desta variável do select. Caso seja necessário voltar , deve ser visto no Gitlab como estava antes da data de  09/08/2024 */
        $dtfilterdata = '';
        $dtfilterdataMed = '';
        if (($dtinicompet != null) && ($dtfincompet != null))  {
            $dtfilterdata = "AND ((TRUNC(FAT.DTINICOMPET) >= TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) AND (TRUNC(FAT.DTFINCOMPET) <= TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')))";
            $dtfilterdataMed = "AND ((TRUNC(MED.DTINICOMPET) >= TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) AND (TRUNC(MED.DTFINCOMPET) <= TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')))";
        };

        if (($dtinicompet != null) && ($dtfincompet == null)) {
            $dtfilterdata = "AND (TRUNC(FAT.DTINICOMPET) >= TO_DATE('{$dtinicompet}', 'DD/MM/YYYY'))";
            $dtfilterdataMed = "AND (TRUNC(MED.DTINICOMPET) >= TO_DATE('{$dtinicompet}', 'DD/MM/YYYY'))";
        };
        
        if (($dtinicompet == null) && ($dtfincompet != null)) {
            $dtfilterdata = "AND (TRUNC(FAT.DTFINCOMPET) <= TO_DATE('{$dtfincompet}', 'DD/MM/YYYY'))";
            $dtfilterdataMed = "AND (TRUNC(MED.DTFINCOMPET) <= TO_DATE('{$dtfincompet}', 'DD/MM/YYYY'))";
        };

        $params = array(
            "NRORG" => $this->Environment->getNrOrg(),
            "NRPARCNEGOCIO" =>  $this->nrParc[0]['NRPARCNEGOCIO']
        );
        $rows = $this->FaturamentoFactory->getFaturamento($dtfilterdata, $dtfilterdataMed, $params);

        foreach ($rows['data'] as &$row) {
            $row["NRPROPOSTA"] = !empty($row["NRORCAMENTO"]) ? $row["NRORCAMENTO"] : $row["NRCONTRATOFCT"];
            $row["DSREFERENCIACONT"] = str_replace("\n", " ",  $row["DSREFERENCIACONT"]);
        }

        return $rows;
    }

    public function getFaturamentoValorFixo($param)
    {
        $nrorg =  $this->Environment->getNrOrg();
        $params = array(
            "NRPARCNEGOCIO" => strval($this->nrParc[0]['NRPARCNEGOCIO']),
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'],
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getFaturamentoValorFixo($params);
    }

    public function getFaturamentoNaoAgrupado($param)
    {
        $nrorg =  $this->Environment->getNrOrg();
        $params = array(
            "NRPARCNEGOCIO" => $this->nrParc[0]['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'],
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );

        $rows = $this->FaturamentoFactory->getFaturamentoNaoAgrupado($params);

        foreach ($rows['data'] as &$pai) {
            foreach ($rows['data'] as $filhos) {
                if ($pai['IDARVOREPAI'] == NULL && $filhos['IDARVOREPAI'] != $pai['IDARVOREPAI']) {
                    if (($pai["VRDESCITEM"] == 0) or ($pai["VRDESCITEM"] == null)) {
                        $pai["VRDESCITEM"] =  NULL;
                    }
                    $pai["QTMINIMA"] = NULL;
                } else {
                    if (($pai["VRDESCITEM"] == 0) or ($pai["VRDESCITEM"] == null)) {
                        $pai["VRDESCITEM"] =  NULL;
                    }
                }
            }
        }
        return $rows;
    }

    public function getItensValorAgrupado($param)
    {
        $nrorg =  $this->Environment->getNrOrg();
        $params = array(
            "NRPARCNEGOCIO" => $this->nrParc[0]['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'],
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        $rows = $this->formatarArvoreGrid($this->FaturamentoFactory->getItensValorAgrupado($params));
        return $rows;
    }

    public function formatarArvoreGrid($rows)
    {
        foreach ($rows['data'] as &$pai) {
            foreach ($rows['data'] as $filhos) {
                if ($pai['IDARVOREPAI'] == NULL && $filhos['IDARVOREPAI'] != $pai['IDARVOREPAI']) {
                    $pai["VRDESCITEM"] =  NULL;
                    $pai["QTMINIMA"] = NULL;
                } else if ($pai['IDARVOREPAI'] == NULL && $filhos['IDARVOREPAI'] != $pai['IDARVOREPAI']) {
                    $pai["QTMINIMA"] = NULL;
                }
            }
        }
        return $rows;
    }

    public function getItensValorAgrupadoEstimado($param)
    {
        $nrorg =  $this->Environment->getNrOrg();
        $params = array(
            "NRPARCNEGOCIO" => $this->nrParc[0]['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'],
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getItensValorAgrupadoEstimado($params);
    }

    public function findBilletIssuedFct($params)
    {
        $param = array(
            "NRLANCTONF" => $params['NR']
        );
        $parametros = $this->FaturamentoFactory->findBilletIssuedFct($param);
        $cdfilial = $parametros[0]['CDFILIAL'];
        $nrlanbolrec = $parametros[0]['NRLANBOLREC'];
        $cdreducont = $parametros[0]['CDREDUCONT'];
        $cdbanco = $parametros[0]['CDBANCO'];
        return $this->BilletIssued->printBillet($cdfilial, $nrlanbolrec, $cdreducont, $cdbanco);
    }

    public function getMedidasVariaveis($param)
    {
        $nrProduto = $param[0]['value'];


        if (substr($nrProduto, 0, 1) === 'G') {
            $arrayMedidas =  $this->FaturamentoFactory->getMedidasVariaveis(array(
                "NRSEQCONTRATO"     => $param[4]['value'],
                "DTINICOMPET"       => $param[2]['value'],
                "DTFINCOMPET"       => $param[3]['value'],
                "IDELEMENTORAIZ"    => $nrProduto
            ));

            // Formatando retorno para que os nomes dos índices estejam em camelCase, conforme esperado pelo frontend. ex: measureName ao invés de MEASURENAME.
            $arrayformatado = array();

            foreach ($arrayMedidas['data'][0] as $key => $value) {
                if ($key == 'INITCOMPETENCE') {
                    $arrayformatado['data'][0]['initCompetence'] = $value;
                } elseif ($key == 'ENDCOMPETENCE') {
                    $arrayformatado['data'][0]['endCompetence'] = $value;
                } elseif ($key == 'MEASURENAME') {
                    $arrayformatado['data'][0]['measureName'] = $value;
                } elseif ($key == 'RNUM') {
                    $arrayformatado['data'][0][$key] = $value;
                } elseif ($key == 'TOTAL_LINES') {
                    $arrayformatado['data'][0][$key] = $value;
                } else {
                    $arrayformatado['data'][0][strtolower($key)] = $value;
                }
            }

            $arrayformatado['pagination'] = $arrayMedidas['pagination'];
            return $arrayformatado;
        } else {

            /* Recebe o valor faturado do front e retira do array de critérios do Mongo */
            $qtFatura = $param[5]['value'];
            unset($param[5]);

            $nrProdutoId    = substr($param[0]['value'], 2, strlen($param[0]['value']));
            $dataSourceName = null;
            $orderBy        = null;
            $cdServOrca     = $param[1]['value'];
            $page           = isset($param['page']) ? intval($param['page']) : '';
            $pageSize       = isset($param['limit']) ? intval($param['limit'])  : '';
    
            $arrayFormatado = $this->criarArrayformatado($param);
            $conditions = $this->filtraElementosArray($arrayFormatado);
            $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
            $repositoryOmProduto = InstanceProvider::getRepository()->getOmProduto();
            $conditions = $conditionsFixed['CONDITIONS'];
            $zmartFilterCriteria = InstanceProvider::getUtilities()->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
    
            $organizacao = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrparcnegocio' => $this->nrParc[0]['NRPARCNEGOCIO']));
    
            if (empty($nrProdutoId)) {
                $productId = '0';
                $zmartFilterCriteria->addCondition('serviceId', '=', $cdServOrca);
            } else {
                $produto   = $repositoryOmProduto->find($nrProdutoId);
                $productId = strval($produto->getNrexternalProductId());
            }
    
            $zmartFilterCriteria->addCondition('productId', '=', $productId);
            $zmartFilterCriteria->setPage($page);
            $zmartFilterCriteria->setPageSize($pageSize);
    
            if (empty($orderBy)) {
                $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
            } else {
                foreach ($orderBy as $idx => $order) {
                    $zmartFilterCriteria->addOrderBy($idx, $order);
                }
            }
    
            $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
            $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
            $registers = InstanceProvider::getLicenseMongoDB()->aggregate("VariableMeasure_{$organizacao->getNrorg()}", $criteria);
    
            foreach ($registers as &$register) {
                
                $register['date'] = \DateTime::createFromFormat('Ymd', $register['date'])->format('d/m/Y');
                $register['endCompetence']  = \DateTime::createFromFormat('Ymd', $register['endCompetence'])->format('d/m/Y');
                $register['initCompetence'] = \DateTime::createFromFormat('Ymd', $register['initCompetence'])->format('d/m/Y');
                $contractId = Utilities::getValueInFilterDataSourceByName($conditions, 'contractId');
                $register['count'] = isset($register['count']) ? $register['count']  : '0';
                $register['countMin'] = $repositoryOmProduto->getMinContract($contractId, $nrProdutoId, $cdServOrca);
                $register['qtFatura'] = $qtFatura;
            }
    
            // $teste = '{"_id":{"$oid":"64c8c9adef5b592dc6474128"},"contractId":"13473","endCompetence":"31/08/2023","id":"43","initCompetence":"01/08/2023","productId":"4476","productName":"SaaS","operatorCode":"000000000997","operatorName":"Teknisa","measureCode":"127","measureName":"Megabyte (Limite)","contractRef":"Odhen Express","contractExpId":"296047","serviceName":null,"serviceId":"4981","date":"15/08/2023","lastDateSuccess":null,"log":[{"NOME_CLIENTE":"A CASA SERENA","SCHEMA_SAAS":"USR_ORG_4172","ESPACO_OCUPADO":"1891"}],"count":"1891","error":null,"lastStatus":"S","countMin":"30000"}';
            // $[json_decode($teste, true)];
    
            $retorno = (object) array(
                'data' =>  $registers,
                'pagination' => (object) array('page' => $page, 'limit' => $pageSize)
            );
            return $retorno;
        }
    }


    public function criarArrayformatado($conditions)
    {
        $retorno = [];
        foreach ($conditions as  $value) {
            if (is_array($value)) {
                array_push($retorno, $value);
            }
        }
        return $retorno;
    }

    public function filtraElementosArray($param)
    {
        $conditions = $param;
        foreach ($conditions as $key => $value) {
            if ($value['columnName'] == 'NRPRODUTO_ID' || $value['columnName'] == 'NRORG_ID' || $value['columnName'] == 'CDSERVORCA') {
                unset($conditions[$key]);
            }
        }
        return $conditions;
    }

    private function fixConditionsToZmartFilterCriteria($conditions)
    {
        $mappedLikeAll = null;
        foreach ($conditions as $idx => &$condition) {
            $name = isset($condition["name"]) ? $condition["name"] : $condition["columnName"];
            if ($condition['operator'] == 'LIKE_I' || $condition['operator'] == 'LIKE') {
                $condition['operator'] = 'LIKE_ALL';
            }
            if ($name == 'date') {
                $condition['value'] = array(
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][0])->format('Ymd')),
                    intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][1])->format('Ymd'))
                );
            } else if ($name == 'initCompetence' || $name == 'endCompetence') {
                $condition['value'] = intval(\DateTime::createFromFormat('d/m/Y', $condition['value'])->format('Ymd'));
            }
            if ($condition['operator'] == 'MAPPED_LIKE_ALL') {
                $mappedLikeAll = $condition;
                unset($conditions[$idx]);
            }
        }
        return array('CONDITIONS' => $conditions, 'MAPPED_LIKE_ALL' => $mappedLikeAll);
    }

    private function setMappedLikeAllCriteria(&$criteria, $mappedLikeAll)
    {
        if (!empty($mappedLikeAll)) {
            if (!isset($criteria[0]['$match'])) {
                $criteria['$match'] = array();
            }
            $name = isset($mappedLikeAll['name']) ? $mappedLikeAll['name'] : $mappedLikeAll['columnName'];
            $mappedLikeAllColumns = array('measureName', 'contractId', 'count', 'operatorName', 'serviceName');
            $value = str_replace('%', '', $mappedLikeAll['value'][0]['_ALL']);
            foreach ($mappedLikeAllColumns as $column) {
                $criteria[0]['$match']['$or'][] = array($column => array('$regex' => ".*{$value}.*"));
            }
        }
    }

    public function getParcelasValorFixo($param)
    {
        $nrorg =  $this->Environment->getNrOrg();
        $params = array(
            "NRSEQCONTRATO" => $param['NRSEQCONTRATO'],
            "NRFATURACONTPAI" => $param['NRFATURACONT'],
            "NRORG"         => $nrorg
        );
        return $this->FaturamentoFactory->getParcelasValorFixo($params);
    }
}
